from analyses.main import analysesmodules
from django.contrib.auth.models import User
from django.core.management.base import BaseCommand, CommandError
from django.db import transaction
from utils.exchange import exchange

class Command(BaseCommand):
    help = "Replaces all system suggestion lists."
    args = "fixture [fixture ...]"

    @transaction.commit_on_success
    def handle(self, *fixture_labels, **options):
        output = []
        try:
            user = User.objects.get(username='__SYSTEM__')
        except User.DoesNotExist:
            raise CommandError("System user not found")
        for module in analysesmodules.get_items_sorted():
            for item in module['appconfig'].get_model('suggestionlist').objects.filter(system=True):
                item.delete(user, override=True)
            for item in module['appconfig'].get_model('apsuggestionlist').objects.filter(system=True):
                item.delete(user, override=True)
        for fixture_label in fixture_labels:
            load = True
            try:
                fixture = open(fixture_label)
            except:
                output.append('Error opening fixture %s' % fixture_label)
                load = False
            if load:
                try:
                    exchange.load(user, fixture.read(), set_dirty=False)
                except:
                    output.append('Error loading fixture %s' % fixture_label)
                fixture.close()
        return '\n'.join(output)
