# user views

from django.contrib.auth import forms as auth_forms
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext_lazy as _

from users import forms as app_forms
from users import models as app_models
from users import tables as app_tables
from utils import crud
from utils.autocomplete import autocomplete

class MixIn(object):
    "Common items for object views"
    model = app_models.User

    def add_to_extra_context(self, extra_context, object):
        extra_context['navigationlist'] =  users_navigationlist()

class display(MixIn, crud.DisplayObjectView):
    form_class = app_forms.UserSheet
    def add_to_extra_context(self, extra_context, object):
        extra_context['navigationlist'] =  users_navigationlist()
        if object.may_change(self.request.user):
            extra_context['defaultbuttons'].append({'url': object.get_change_password_url(), 'label': _('Change password')})

class edit(MixIn, crud.EditObjectView):
    form_class = app_forms.UserForm
    def get_redirect(self, refform, object):
        return HttpResponseRedirect(object.get_absolute_url())

class set_password(MixIn, crud.EditObjectView):

#TODO: send mail to user to inform his password was changed by the administrator

    form_class = auth_forms.SetPasswordForm
    def get_form(self, object, **formargs):
        """
        Special because SetPasswordForm is not a modelform
        """
        if self.request.method == 'POST':
            form = self.form_class(object, data=self.request.POST, files=self.request.FILES, **formargs)
        else:
            form = self.form_class(object, initial=self.get_initial(object), **formargs)
        self.setup_form(form, object)
        return form

    def save_form(self, form):
        object = form.save()
        return object

    def get_redirect(self, refform, object):
        return HttpResponseRedirect(object.get_absolute_url())

class change_password(set_password):
    """
    View to allow the user to change her password
    """
    form_class = auth_forms.PasswordChangeForm
    objectpermission = None
    def __call__(self, request, *args, **kwargs):
        """
        No need for object_id, we'll get the user object from the request
        """
        return super(change_password, self).__call__(request, *args, **kwargs)

    def lookup_object(self,*args, **kwargs):
        return self.request.user

    def get_redirect(self, refform, object):
        return HttpResponseRedirect('/')

    def add_to_extra_context(self, extra_context, object):
        # no navigationlist here
        pass

class delete(MixIn, crud.DeleteObjectView):
    form_class = app_forms.UserSheet

    def get_default_url(self, object):
        return reverse('users_users')

class add(MixIn, crud.AddObjectView):
    form_class = auth_forms.UserCreationForm
    def get_redirect(self, refform, object):
        return HttpResponseRedirect(object.get_edit_url())

class UserFilter(crud.Filter):
    form = app_forms.UserFilterForm
    filters = [
                {'filteritem': 'username', 'parameter': 'username__icontains', 'type': 'normal'},
            ]

class users(crud.TableListView):
    "List of users"

    table_class = app_tables.UsersTable
    defaultsort = 'username'
    queryset = app_models.User.objects.all().order_by('username')
    paginate_by = 100
    title = _('Users')
    filter = UserFilter()
    redirect_on_unique = True

    def add_to_extra_context(self, extra_context):
        super(users, self).add_to_extra_context(extra_context)
        extra_context['navigationlist'] =  users_navigationlist('users')
        if app_models.User.may_add(self.request.user):
            extra_context['tablebuttons'] = [{
                    'url': reverse('users_user_add'),
                    'label': _('Add'),
                }]

def users_navigationlist(current=None):
    navigationlist = [
        {'name': 'users',
            'url': reverse('users_users'),
            'label': _('Users')},
        {'name': 'groups',
            'url': reverse('users_groups'),
            'label': _('Groups')},
    ]
    nav = []
    for item in navigationlist:
        if current == item['name']:
            nav.append({'label': item['label']})
        else:
            nav.append({'label': item['label'], 'url': item['url']})
    return nav
